# -*- coding: utf-8 -*-

# -----------------------------------------------------
#
#	|                         o|   |          |2011
#	|,---.,---.,---.,-.-.,---..|---|,---.,---.|__/ 
#	||   |`---.|   || | ||   |||   |,---||    |  \ 
#	``   '`---'`---'` ' '`   '``   '`---^`---'`   `
#
# -----------------------------------------------------
# (c) SCRT, 2010 / author: ~SaD~ / Last mod: 03.03.11
# -----------------------------------------------------
# 
# Run challenge:
# -------------------
# $> python crypto2.py
# 
# Runs the (pseudo) HTTP server on address/port defined below.
#
# -----------------------------------------------------

from crypto2_utils import *

import re
import random
from urllib import quote, unquote
from BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer

__serverAddress__ = 'localhost'
#__serverAddress__ = 'ep2.insomni.hack'
__serverPort__ = 8080
__secret3DESKey__ = '03kjq3wksdjsd-9q3%.*0993'

__rxpExtractClear__ = re.compile(r'[&|?]t=(?P<text>.[^&?#]*)', re.IGNORECASE)
__rxpExtractCipher__ = re.compile(r'[&|?]t=(?P<text>[a-fA-F0-9]*)\b', re.IGNORECASE)


class OracleHTTPHandler(BaseHTTPRequestHandler):
	
	server_version = 'InsomniHack/2011'
	sys_version = ''
	
	def do_POST(self):
		self.resp_genericError('Fonction non supportée!')
	
	def do_HEAD(self):
		self.resp_genericError('Fonction non supportée!')

	def do_GET(self):
		
		if len(self.path) > 255:
			self.resp_genericError('URL trop long!')
			
		else:
			if unquote(self.path).startswith("/enc/?"):
				if len(__rxpExtractClear__.findall(unquote(self.path))) > 0 and __rxpExtractClear__.findall(unquote(self.path))[0] != '':
				
					clear = __rxpExtractClear__.findall(unquote(self.path))[0]
					iv, cipher = self.encrypt(__rxpExtractClear__.findall(unquote(self.path))[0])
					
					self.resp_encryptResult(clear, iv, cipher)
				else:
					self.resp_homePage()
			
			elif unquote(self.path).startswith("/dec/?"):
				if len(__rxpExtractCipher__.findall(unquote(self.path))) > 0 and __rxpExtractCipher__.findall(unquote(self.path))[0] != '':
					rawCipher = __rxpExtractCipher__.findall(unquote(self.path))[0]
					
					if len(rawCipher) % 4 != 0 or len(rawCipher) % 16 != 0:
						self.resp_genericError("Seule une chaine d'octets, de longeur multiple de 8 (octets) est acceptée!")
					else:
						iv = HexListToBinString(toBytes(rawCipher[:16]))
						cipher = HexListToBinString(toBytes(rawCipher[16:]))
						try:
							if self.decrypt(iv, cipher) != None:
								self.resp_decryptResult(rawCipher, 'Déchiffrement réussi!')
						except RuntimeError as e:
							self.resp_decryptResult(rawCipher, 'Déchiffrement échoué! [Padding Error]')
				else:
					self.resp_homePage()
			else:
				self.resp_homePage()
	
	
	def decrypt(self, iv, cipher):
		c = triple_des(__secret3DESKey__, CBC, iv, pad=None, padmode=PAD_PKCS5)
		return c.decrypt(cipher)
	
	def encrypt(self, clear):
		iv = self.freshIV()
		c = triple_des(__secret3DESKey__, CBC, iv, pad=None, padmode=PAD_PKCS5)
		return (iv, c.encrypt(clear))
	
	def freshIV(self):
		iv = []
		for i in range(0, 8):
			iv.append(random.choice(range(0x00, 0x100)))
		return HexListToBinString(iv)
	
	def resp_homePage(self):
		self.send_response(200)
		self.send_header('Content-type', 'text/plain; charset=utf-8')
		self.end_headers()
		self.wfile.write("\
%s\n\nBienvenue sur le serveur de chiffrement sécurisé de Insomni'Hack 2011 !\n\
Il permet de chiffrer un texte en 3DES (mode CBC) et d'en vérifier le déchiffrement correct.\n\
Il ne permet toutefois *PAS* d'avoir accès au texte déchiffré ni à la clé de chiffrement.\n\n\
Pour chiffrer du texte, utilisez l'URL : http://%s:%d/enc/?t=[votre_texte]\n\
Pour vérifier le déchiffrement, utilisez l'URL : http://%s:%d/dec/?t=[resultat_du_chiffrement]\n\n\
EXEMPLE ---------------------------------------------\n\
http://%s:%d/enc/?t=insomni'Hack2011\n\
http://%s:%d/dec/?t=3F3FCA8CF9F6FF12A676353965A00ECECB2D34857370403375C5BE25ED5A5210\n\
-----------------------------------------------------\
" % (insHeader(), __serverAddress__, __serverPort__, __serverAddress__, __serverPort__, __serverAddress__, __serverPort__, __serverAddress__, __serverPort__))
		return
	
	def resp_decryptResult(self, cipher, status):
		self.send_response(200)
		self.send_header('Content-type', 'text/plain; charset=utf-8')
		self.end_headers()
		self.wfile.write('%s\n\n3DES(texte):\t%s [IV = %s]\nRésultat:\t%s' % (insHeader(), cipher, cipher[:16], status))
		return
	
	def resp_encryptResult(self, clear, iv, cipher):
		self.send_response(200)
		self.send_header('Content-type', 'text/plain; charset=utf-8')
		self.end_headers()
		self.wfile.write('%s\n\nTexte:\t\t%s\n3DES(texte):\t%s' % (insHeader(), clear, toHexString(bs2hl(iv + cipher), 1)))
		return
	
	def resp_genericError(self, error):
		self.send_response(200)
		self.send_header('Content-type', 'text/plain; charset=utf-8')
		self.end_headers()
		self.wfile.write('%s\n\nErreur:\t%s' % (insHeader(), error))
		return

def insHeader():
	return """\
 -----------------------------------------------------
  |                         o|   |          |2011
  |,---.,---.,---.,-.-.,---..|---|,---.,---.|__/ 
  ||   |`---.|   || | ||   |||   |,---||    |  \ 
  ``   '`---'`---'` ' '`   '``   '`---^`---'`   `
 -----------------------------------------------------"""

def main():
	try:
		server = HTTPServer((__serverAddress__, __serverPort__), OracleHTTPHandler)
		print ''
		print insHeader()
		print ''
		print '[*] Crypto/Padding_Oracle [by SaD]'
		print '[*] Server listening on %s:%d' % (__serverAddress__, __serverPort__)
		server.serve_forever()
	except KeyboardInterrupt:
		print '[*] Stopping server ...'
		server.socket.close()

if __name__ == '__main__':
	main()